-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009 - 2015 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

-- Generic requires for racial based on talent level
undeads_req1 = {
	level = function(level) return 0 + (level-1)  end,
}
undeads_req2 = {
	level = function(level) return 8 + (level-1)  end,
}
undeads_req3 = {
	level = function(level) return 16 + (level-1)  end,
}
undeads_req4 = {
	level = function(level) return 24 + (level-1)  end,
}

newTalentType{ type="undead/abomination", name = "abomination", generic = true, description = "Abomination's innate abilities."}

local Stats = require "engine.interface.ActorStats"

-- [[1: Redundant Organs
-- Passive
-- Flavour text: Your unholy amalgamation of different body parts is composed by several redundant vital organs and nervous systems. 
-- As such, common ailments that compromise individual systems are less effective on you, as you quickly remove the damaged or tainted 
-- parts and replace them using the corpses of your foes. The extra amount of lungs also allow you to stay submerged for far longer than
-- ordinary individual creatures.
-- - large poison resistance bonus
-- - medium disease resistance bonus
-- - small stun resistance bonus
-- - medium extra air bonus
-- - small 'only die below -x HP' bonus
-- - +1.0 life regen per point
-- - The first point in Redundant Organs increases your healing factor by 3% for every size category above 'Medium'

newTalent{
        short_name = 'REDUNDANT_ORGANS',
        name = "Redundant Organs",
        type = {"undead/abomination", 1},
        mode = "passive",
        require = undeads_req1,
        points = 5,
        mode = "passive",
                getDisease_Immune = function(self, t) return self:getTalentLevel(t) * 0.1 end,
                getPoison_Immune = function(self, t) return self:getTalentLevel(t) * 0.15 end,
                getStun_Immune = function(self, t) return self:getTalentLevel(t) * 0.05 end,
                getMax_Air = function(self, t) return self:getTalentLevel(t) * 10 end,
                getDie_At = function(self, t) return self:getTalentLevel(t) * -30 end,
				getLife_Regen = function(self, t) return self:getTalentLevel(t) * 1 end,
                passives = function(self, t, p)
                        self:talentTemporaryValue(p, "disease_immune", t.getDisease_Immune(self, t))
                        self:talentTemporaryValue(p, "poison_immune", t.getPoison_Immune(self, t))
                        self:talentTemporaryValue(p, "stun_immune", t.getStun_Immune(self, t))
                        self:talentTemporaryValue(p, "max_air", t.getMax_Air(self, t))
                        self:talentTemporaryValue(p, "die_at", t.getDie_At(self, t))
						self:talentTemporaryValue(p, "life_regen", t.getLife_Regen(self, t))
                end,
        on_learn = function(self, t)
                self:callTalent(t.id, "callbackOnAct") 
        end,
        on_unlearn = function(self, t)
                self:callTalent(t.id, "callbackOnAct")
        end,
        callbackOnAct = function(self, t)
                if self.abomination_healing then
                        self:removeTemporaryValue("healing_factor", self.abomination_healing)
                end
                local mod = math.max(0, (self.size_category or 0) - 3)
                local talents = self:numberKnownTalent("undead/abomination")
                local healing_factor = mod * talents * .03
                self.abomination_healing = self:addTemporaryValue("healing_factor", healing_factor)
        end,
        info =  function(self, t)
                local  diseaseImmune, poisonImmune, stunImmune, air, dieAt = t.getDisease_Immune(self, t)*100, t.getPoison_Immune(self, t)*100, t.getStun_Immune(self, t)*100, t.getMax_Air(self, t), t.getDie_At(self, t)
                return ([[Your unholy amalgamation of different body parts is composed of several redundant vital organs and nervous systems. Common ailments that compromise individual systems are less effective on you, as you quickly remove the damaged or tainted parts and replace them using the corpses of your foes. The extra amount of lungs also allow you to stay submerged for far longer than ordinary individual creatures.
                               
Increases disease resistance by %d%%.
Increases poison resistance by %d%%.
Increases stun resistance by %d%%.
Increases your maximum air by %d.
Only die at %d life. However, when below 0 HP, you cannot see how much life you have left.
                               
Each point in Redundant Organs increases your base health regeneration by 1.0 and the first point in Redundant Organs increases your healing factor by 3%% for every size category above 'Medium']]):format(diseaseImmune, poisonImmune, stunImmune, air, dieAt)
				end
}

-- 2: Millipede
-- Passive
-- Flavour text: As the multitude of fallen minds that compose you harmonizes, you learn better how to move your extra pairs of legs more
-- naturally and graciously, allowing you to attach even more such appendages to yourself as your prowess grows. Additionally, you can now tear away any appendages holding you in place, freeing yourself, but also damaging yourself in the process.
-- - grants an activatable instant ability to remove a pinning effect in exchange for bleeding X health over Y turns (health amount decreases with talent level, bleed duration increases with size category)
-- - 20% off-balance immunity per level
-- - Medium movement speed bonus (8% per talent level)
-- - small global speed penalty (3% per talent level)
-- - +1 extra size category at levels 2 and 4
-- - MAYBE 1 extra boots slot at talent level 6 (needs category point investment)
-- - +1.0 life regen per point
-- - The first point in Millipede increases your healing factor by 3% for every size category above 'Medium'

newTalent{
	short_name = 'MILLIPEDE',
	name = "Millipede",
	type = {"undead/abomination", 2},
	require = undeads_req2,
	points = 5,
	no_unlearn_last = true,
	mode = "passive",	
                getGlobal_Speed = function(self, t) return self:getTalentLevel(t) * -0.03 end,
                getMovement_Speed = function(self, t) return self:getTalentLevel(t) * 0.08 end,
				getLife_Regen = function(self, t) return self:getTalentLevel(t) * 1 end,
				getChance = function(self, t) return self:getTalentLevelRaw(t) * 20 end,
				
                passives = function(self, t, p)
                        self:talentTemporaryValue(p, "global_speed_add", t.getGlobal_Speed(self, t))
                        self:talentTemporaryValue(p, "movement_speed", t.getMovement_Speed(self, t))
						self:talentTemporaryValue(p, "life_regen", t.getLife_Regen(self, t))
						
				local millipedesize1 = 0
				local millipedesize2 = 0
				local millipedesize3 = 0
				if self:getTalentLevel(t) >= 2 then millipedesize1 = 1 end
				self:talentTemporaryValue(p, "size_category", millipedesize1)
				
				if self:getTalentLevel(t) >= 4 then millipedesize2 = 1 end
				self:talentTemporaryValue(p, "size_category", millipedesize2)
	
				if self:getTalentLevel(t) >= 6 then millipedesize3 = 1 end
				self:talentTemporaryValue(p, "size_category", millipedesize3)
				
-- 				callbackOnTemporaryEffect = function(self, t, eff_id, e, p) -- this should be defined as it's own field
-- 					if eff_id=="EFF_OFFBALANCE" then
-- 					local chance = self:callTalent(self.T_MILLIPEDE, "getChance")
-- 						if rng.percent(chance) then
-- 						p.dur=-1000 --in case any other callbacks try to /add/ to it
-- 						end
-- 					end
-- 				end
				
					--Extra boots slot -- this will have to happen in two parts; we will unlock it with a temp value set in on_learn and on on_unlearn, and then actually make the change with a callbackOnActBase, to ensure it happens /outside/ of the level up dialog. once these changes are made, they are basically impossible to reverse, so you could put points in, take them out, and still get the slot if it was handled the old way.
-- 					if self:getTalentLevel(t) >= 6 then
-- 						if self.body then
--                               self.body.FEET = 2
--                         else
--                               self.body = { FEET = 2 }
--                         end
-- 						self:initBody()
-- 					end
                end,
				
        on_learn = function(self, t)
					self.temp_xtra_feet = self.temp_xtra_feet or 0
				if self:getTalentLevel(t) >= 1 and not self:knowTalent(self.T_YANK_OUT_LEGS) then
			self:learnTalent(self.T_YANK_OUT_LEGS, true)
				end
 --               self:callTalent(t.id, "callbackOnAct")
						if self:getTalentLevel(t) >= 6 and self.temp_xtra_feet == 0 then
							self.temp_xtra_feet = 1
						end
        end,
        on_unlearn = function(self, t)
				if self:getTalentLevel(t) < 1 and self:knowTalent(self.T_YANK_OUT_LEGS) then
			self:unlearnTalent(self.T_YANK_OUT_LEGS)
				end
				if self:getTalentLevel(t) < 6 then
					self.temp_xtra_feet = 0
				end
  --              self:callTalent(t.id, "callbackOnAct")
        end,
				callbackOnTemporaryEffect = function(self, t, eff_id, e, p) -- I would be quite surprised if this was actually working inside of 'passive'. the engine is going to look for a talent with this field defined, so it won't find it inside of another field.
					if eff_id=="EFF_OFFBALANCE" then
					local chance = self:callTalent(self.T_MILLIPEDE, "getChance")
						if rng.percent(chance) then
						p.dur=-1000 --in case any other callbacks try to /add/ to it
						end
					end
				end,
				callbackOnActBase = function(self, t) -- this has to happen a turn after the level up dialog closes. if it happens /in/ the level up dialog, it is next to impossible to remove the extra leg slot if they take the point out. we gate it with the temp value so it will only ever happen once.
					if self:getTalentLevel(t) >= 6 and self.temp_xtra_feet == 1 then
						self.temp_xtra_feet = 2 -- basically blocks this callback from ever firing again, even if the player changes their mastery with a necklace or such
						local boot = self:getInven("FEET") and self:getInven("FEET")[1] or false -- when we run initBody, it's going to completely initialize body.FEET, effectively erasing the old slot. we'll look for boots first and take them off before we initialize to avoid them vanishing.
						if boot then
							self:doTakeoff("FEET", 1, boot, false, self)
						end
						if self.body then
							self.body.FEET = 2
						else
							self.body = { FEET = 2 }
						end
						self:initBody()
					end
				end,

        info =  function(self, t)
				local GlobalSpeed, MovementSpeed, Chance = t.getGlobal_Speed(self, t) * 100, t.getMovement_Speed(self, t) * 100, t.getChance(self, t)
                return ([[As the multitude of fallen minds that compose you harmonizes, you learn to better coordinate your extra pairs of legs more naturally and graciously, allowing you to attach even more such appendages to yourself as your prowess grows. Additionally, you can now tear away any appendages holding you in place, freeing yourself, but also damaging yourself in the process. However, by doing so you grow in size and become more sluggish.

Decreases global speed by %d%%.
Increases movement speed by %d%%.
Increases off-balance resistance by %d%%.
Increases Size Category by 1 for every 2 soft talent levels.	

At talent level 6, you get an extra boots slot. (WARNING: This takes a moment (one turn must pass), and your boots will be unequipped; you should do this in safety.)
				
Each point in Millipede increases your base health regeneration by 1.0 and the first point in Millipede increases your healing factor by 3%% for every size category above 'Medium']]):format(GlobalSpeed, MovementSpeed, Chance)
				end
}

-- 3: Assimilate
-- Active
-- Flavour text: You learn to shed off your damaged and incapacitated parts and incorporate your fallen enemies to yourself during battle.
-- - Heal X HP for each enemy killed for the next Y turns (heal amount scales with size category, durations with Con)
-- - Convert X% of damage into bleeding damage over Y amount of turns, for Z turns (damage % scales with Con, smearing length scales with size category, effect duration scales with talent level)
-- - at talent level 5 becomes an instant action
-- (Passively)
-- - +1.0 life regen per point
-- - The first point in Assimilate increases your healing factor by 3% for every size category above 'Medium'

newTalent{
	short_name = 'ASSIMILATE',
	name = "Assimilate",
	type = {"undead/abomination", 3},
	require = undeads_req3,
	points = 5,
	mode = "activated",
	no_energy = function(self, t) if self:getTalentLevel(t) >= 5 then return true end return false end,
	cooldown = function(self, t) return math.min( 30 - self:getTalentLevel(t) * 2, 30 - ((self.size_category or 0) - 3) * 3) end,
	range = 0,
	requires_target = false,
	
				getLife_Regen = function(self, t) return self:getTalentLevel(t) * 1 end,
				getPercent = function(self, t) return 10 + self:getTalentLevel(t) + (self:getCon() / 3)  end, 			--Smeared Damage %
				getDuration1 = function(self, t) return 3 + ((self.size_category or 0) - 3) end,						--Smeared over X turns
				getBuff_Duration = function(self, t) return self:getTalentLevelRaw(t) * 2 + 2 end,						--Smearing buff duration
				getHeal_Amount = function(self, t) return ((self.size_category or 0) - 3) * 25 + 50 end,				--Heal per kill
				getDuration2 = function(self, t) return 2 + (self:getCon() / 5 ) end,									--Heal per kill buff duration
				
                passives = function(self, t, p)
					self:talentTemporaryValue(p, "life_regen", t.getLife_Regen(self, t))
                end,
	
	action = function(self, t)
		self:setEffect(self.EFF_ASSIMILATE_STRIP, t.getBuff_Duration(self, t), {smear_percent=t.getPercent(self, t), smear_dur=t.getDuration1(self, t)} )
		self:setEffect(self.EFF_ASSIMILATE_DEAD, t.getDuration2(self, t), {hp_per_kill=t.getHeal_Amount(self,t)})
		return true
		
	end,
	
    info =  function(self, t)
				local DamagePercent, BleedDuration, SmearDuration, HealAmount, HealDuration = t.getPercent(self, t), t.getDuration1(self, t), t.getBuff_Duration(self, t), t.getHeal_Amount(self, t), t.getDuration2(self, t)
                return ([[You learn to shed off your damaged and incapacitated parts and incorporate your fallen enemies to yourself during battle.
				
				You convert %d%% of damage into bleeding damage over %d turns, for %d turns. Amount of damage converted scales with Constitution, amount of turns over which the bleed damage becomes smeared scales with size category and buff duration increases with talent level.
				You also heal %d hit points per kill for %d turns. Amount of hit points recovered per kill scales with size category, while buff duration scales with Constitution.
				
				Cooldown decreases with higher size categories. At raw talent level 5 it becomes an instant action.
				
				Each point in Assimilate increases your base health regeneration by 1.0 and the first point in Assimilate increases your healing factor by 3%% for every size category above 'Medium']]):format(DamagePercent, BleedDuration, SmearDuration, HealAmount, HealDuration)
				end,
}
		
-- 4: Hecatonchires
-- Passive
-- Flavour text: You further master the melding of fallen minds with your own and the magical creation of necessary nerve endings, vastly increasing your proficiency with the arms, tentacles and other such appendages you assimilate from your fallen enemies.
-- Additionally, you can now tear away any appendages rendered unable to hold your weapons, rearming yourself, but also damaging yourself in the process.
-- - small chance for bonus unarmed attack (up to 50% damage) on random adjacent enemies (1 per turn, 2 per turn on talent level 5) 
-- - small chance for bonus disarm move (no damage) on 1 random adjacent target
-- - small chance to grapple random adjacent enemy, inflicting grapple debuffs, but no damage. Breaks on movement
-- - grants an activatable instant ability to remove a disarm effect in exchange for bleeding X health over Y turns (health amount decreases with talent level, bleed duration increases with size category)
-- - small global speed penalty (3% per talent level)
-- - +1 extra size category at levels 2 and 4
-- - MAYBE 1 extra gloves slot at talent level 6 (needs category point investment)
-- - AND/OR ability to wield dual-handed weapons with 1 hand at raw level 5
-- - +1.0 life regen per point
-- - The first point in Hecatonchires increases your healing factor by 3% for every size category above 'Medium'

newTalent{
	short_name = 'HECATONCHIRES',
	name = "Hecatonchires",
	type = {"undead/abomination", 4},
	mode = "passive",
	require = undeads_req4,
	points = 5,
	no_unlearn_last = true,
	mode = "passive",
	            getGlobal_Speed = function(self, t) return self:getTalentLevel(t) * -0.03 end,
				getLife_Regen = function(self, t) return self:getTalentLevel(t) * 1 end,
				
				passives = function(self, t, p)
								self:talentTemporaryValue(p, "global_speed_add", t.getGlobal_Speed(self, t))
								self:talentTemporaryValue(p, "life_regen", t.getLife_Regen(self, t))	
	
				local sizeval1 = 0
				local sizeval2 = 0
				local sizeval3 = 0
				if self:getTalentLevel(t) >= 2 then sizeval1 = 1 end
				self:talentTemporaryValue(p, "size_category", sizeval1)
				
				if self:getTalentLevel(t) >= 4 then sizeval2 = 1 end
				self:talentTemporaryValue(p, "size_category", sizeval2)
	
				if self:getTalentLevel(t) >= 6 then sizeval3 = 1 end
				self:talentTemporaryValue(p, "size_category", sizeval3)
	

				end,
				
		on_learn = function(self, t)
				self.temp_xtra_hands = 0
				if self:getTalentLevelRaw(t) == 5 then self:attr("allow_mainhand_2h_in_1h", 1) end
					if self:getTalentLevel(t) >= 1 and not self:knowTalent(self.T_YANK_OUT_ARMS) then
			self:learnTalent(self.T_YANK_OUT_ARMS, true)
					end
				if self:getTalentLevel(t) >= 6 and self.temp_xtra_hands == 0 then
					self.temp_xtra_hands = 1
				end
	--	self:callTalent(t.id, "callbackOnAct")
        end,
		
        on_unlearn = function(self, t)
				if self:getTalentLevelRaw(t) == 4 then self:attr("allow_mainhand_2h_in_1h", -1) end
				if self:getTalentLevel(t) < 1 and self:knowTalent(self.T_YANK_OUT_ARMS) then
					self:unlearnTalent(self.T_YANK_OUT_ARMS)
				end
				if self:getTalentLevel(t) < 6 then
					self.temp_xtra_hands = 0
				end
       -- self:callTalent(t.id, "callbackOnAct")
        end,
			callbackOnActBase = function(self, t) -- this has to happen after the level up dialog closes. if it happens /in/ the level up dialog, it is next to impossible to remove the extra slot if they take the point out. we gate it with the temp value so it will only ever happen once.
					if self:getTalentLevel(t) >= 6 and self.temp_xtra_hands == 1 then
						self.temp_xtra_hands = 2
						local glove = self:getInven("HANDS") and self:getInven("HANDS")[1] or false
						if glove then
							self:doTakeoff("HANDS", 1, glove, false, self)
						end
						if self.body then
							self.body.HANDS = 2
						else
							self.body = { HANDS = 2 }
						end
						self:initBody()
					end
				end,

        info =  function(self, t)
				local GlobalSpeed = t.getGlobal_Speed(self, t) * 100
                return ([[You further master the melding of fallen minds with your own and the magical creation of necessary nerve endings, vastly increasing your proficiency with the arms, tentacles and other such appendages you assimilate from your fallen enemies. Additionally, you can now tear away any appendages rendered unable to hold your weapons, rearming yourself, but also damaging yourself in the process.

Decreases global speed by %d%%.
Increases Size Category by 1 for every 2 soft talent levels.			
				
At raw talent level 5 you can wield two-handed weapons with only one hand.

At soft talent level 6, you get an extra gloves slot. (WARNING: This takes a moment (one turn must pass), and your gloves will be unequipped; you should do this in safety.)
				
Each point in Hecatonchires increases your base health regeneration by 1.0 and the first point in Hecatonchires increases your healing factor by 3%% for every size category above 'Medium']]):format(GlobalSpeed)
				end
}

newTalent{
	short_name = 'YANK_OUT_LEGS',
	name = "Yank Out Legs",
	type = {"undead/other", 1},
	points = 5,
	mode = "activated",
	no_energy = true,
	cooldown = 0,
	range = 0,
	requires_target = false,
	getSelfDamage = function(self, t)
		local level = self.level or 10 --I can't see the user lacking a level, but this /will/ stop Lua errors
		return math.max(50, level * 5 - self:getCon() )
	end,
	getDuration = function (self, t) 
		local dur = 3 + math.max(0, (self.size_category or 0) - 3)
		return dur 
	end,
	on_pre_use = function(self, t, silent)
		local filter = {subtype={pin=true}}
		local no_of_effects = #self:effectsFilter(filter) --returns a list of the effect IDs
		if no_of_effects<1 then
			if not silent then
				game.logPlayer(self, "You must be pinned to use Yank Out Legs!")
			end
			return false
		end
		return true
	end,
	action = function(self, t)
		local filter = {subtype={pin=true}}
		local no_of_effects = self:removeEffectsFilter(filter, 1) --returns the number of effects removed
		local dur = 3 + math.max(0, (self.size_category or 0) - 3)
		if no_of_effects>=1 then
			if self:canBe("cut") then
				self:setEffect(self.EFF_CUT, dur, {src=self, power=t.getSelfDamage(self, t)/dur, no_ct_effect=true})
			end
			return true
		end
	end,   
	info = function(self, t)
		local self_damage, duration = t.getSelfDamage(self, t), t.getDuration(self, t)
		return ([[Violently remove your pinned or immobilized appendages, freeing yourself, at the expense of bleeding %d hit points for %d turns. The damage scales with character level, but will be mitigated by Constitution and the bleed duration increases by 1 for every size category above 'medium'.]]):format(self_damage, duration)
	end
}

newTalent{
	short_name = 'YANK_OUT_ARMS',
	name = "Yank Out Arms",
	type = {"undead/other", 1},
	points = 5,
	mode = "activated",
	no_energy = true,
	cooldown = 0,
	range = 0,
	requires_target = false,
	getSelfDamage = function(self, t)
		local level = self.level or 10 --I can't see the user lacking a level, but this /will/ stop Lua errors
		return math.max(50, level * 5 - self:getCon() )
	end,
	getDuration = function (self, t) 
		local dur = 3 + math.max(0, (self.size_category or 0) - 3)
		return dur 
	end,
	on_pre_use = function(self, t, silent)
		local filter = {subtype={pin=true}}
		local no_of_effects = #self:effectsFilter(filter) --returns a list of the effect IDs
		if no_of_effects<1 then
			if not silent then
				game.logPlayer(self, "You must be disarmed to use Yank Out Arms!")
			end
			return false
		end
		return true
	end,
	action = function(self, t)
		local filter = {subtype={disarm=true}}
		local no_of_effects = self:removeEffectsFilter(filter, 1) --returns the number of effects removed
		local dur = 3 + math.max(0, (self.size_category or 0) - 3)
		if no_of_effects>=1 then
			if self:canBe("cut") then
				self:setEffect(self.EFF_CUT, dur, {src=self, power=t.getSelfDamage(self, t)/dur, no_ct_effect=true})
			end
			return true
		end
	end,   
	info = function(self, t)
		local self_damage, duration = t.getSelfDamage(self, t), t.getDuration(self, t)
		return ([[Violently remove your crippled arms, using your weapon(s) with a different set of arms, at the expense of bleeding %d hit points for %d turns. The damage scales with character level, but will be mitigated by Constitution and the bleed duration increases by 1 for every size category above 'medium'.]]):format(self_damage, duration)
	end
}